# OE1100
# ################################################################################################################
#
# Description: Adds an item search control to the OE Order entry screen. 
#              Use the navigation arrows on the control to find the next item. 
################################################################################################################
# Copyright 2022 Orchid Systems 
#
# Permitted for use with Orchid Systems Modules only.
# The code is supplied on an "as is" basis for training and demonstration purposes only 
# and is not supported by Orchid Systems. If this code is deployed in a live production environment it is the 
# responsibility of the End User to ensure that it is operating as required.
################################################################################################################

from accpac import *

def main(args):
	MyUI()

class MyUI(UI):
	def __init__(self):
		UI.__init__(self)
		self.createScreen()

	def createScreen(self):
		self.getControlInfo("form|cmdShipAll1|stbOrderEntry", self.onControlInfo)
		self.dsH = self.openDataSource("adsOEORDH")
		self.dsH.onAfter = self.dsH_onAfter
		
		self.dsD = self.openDataSource("adsOEORDD")
		self.dsD.onAfter = self.dsD_onAfter
		
		self.onClose = self.checkSaveRecordBeforeClosing
		
		self.show()
		
	def onControlInfo(self, info):
		
		f = info.find("form")
		sstab = info.find("stbOrderEntry")
		cmdShipAll = info.find("cmdShipAll1")
		
		fld = self.addUIField("FINDITEM")
		fld.controlType = "EDIT"
		fld.fieldType = "STRING"
		fld.left = cmdShipAll.left + sstab.left + cmdShipAll.width + 150
		fld.top = cmdShipAll.top + sstab.top - f.height
		fld.width = 5000
		fld.labelWidth = 40
		fld.caption = "Item Search:"
		fld.hasFinder = True
		fld.hasArrows = True
		fld.onClick = self.fldFindItem_onClick
		fld.onChange = self.fldFindItem_onChange
		#fld.setOnKeyDown(self.fldFindItem_onKeyDown)
		fld.tabStop = True
		self.fldFindItem = fld
		
		## Add dummy invisible field.
		## Used so that the search field can be tabbed out of.
		fld = self.addUIField("NOTINUSE")
		fld.controlType = "EDIT"
		fld.fieldType = "STRING"
		fld.left = self.fldFindItem.left + sstab.left + self.fldFindItem.width + 500
		fld.top = -f.height
		fld.width = 0
		fld.labelWidth = 0
		fld.tabStop = True
		self.fldNotInUse = fld
		
		self.getHostControl("stbOrderEntry").setOnClick(self.onTabClick)
		
		self.docnum = self.getHostControl("afeOEORDHordnumber")
		self.docnum.setOnClick(self.docnum_onClick)
		
		grid = self.getHostControlAvl("avlOEORDDdetail1")
		grid.setOnLDblClick(self.onGridLeftClick)
		
		self.show()
		
	def onTabClick(self, tab):
		if tab == 0:
			## show the search field.
			self.fldFindItem.show()
		else:
			## hide the search field.
			self.fldFindItem.hide()
			
	def onGridLeftClick(self, e):
		## Works best on fields that are not editable such as the item field after a detail line has been inserted.
		#showMessageBox("row = [{}], col = [{}]".format(e.row, e.colIndex))
		if e.colIndex == 5:
			## Item column.
			self.fldFindItem.setValue(self.dsD.get("ITEM"))
			
	def dsH_onAfter(self, eventName, field, value):
		if eventName in ["READ", "FETCH", "INIT", "UPDATE", "INSERT"] or eventName.startswith("GO"):
			self.getControlInfo("APP_Save_Button", self.setFieldStatus)
		elif eventName == "OPEN":
			self.getControlInfo("APP_Save_Button", self.setFieldStatus)

		elif eventName == "PUT" and field == "CUSTOMER":
			self.fldFindItem.enable()
			
	def dsD_onAfter(self, eventName, field, value):
		pass
		# if eventName.startswith("KEYGO"): # or eventName.startswith("GO"):
			# self.fldFindItem.setValue(self.dsD.get("ITEM"))
			
	def onAfterOpen(self):
		ctrl = self.getHostControl("FINDITEM")
		ctrl.setOnKeyDown(self.fldFindItem_onKeyDown)
			
	def docnum_onClick(self, btnType):
		if btnType == 5:
			# the new button (+) was pressed.
			self.fldFindItem.setValue("")
			self.fldFindItem.disable()	# should be re-enabled when they enter a customer code.
				
	def checkSaveRecordBeforeClosing(self):
		if self.dsH.dirty() != 0:
			# ask the user to save the record.
			self.message("Do you want to post this order?", "Confirmation", "QUESTION", "YESNOCANCEL", self.onConfirmSave)
			
	def onConfirmSave(self, r):
		if r == "YES":
			self.abortClose()
			self.setTimeout(50, self.saveRecord)
		elif r == "NO":
			pass
		elif r == "CANCEL":
			# abort closing.
			self.abortClose()
			
	def saveRecord(self):
		self.getHostControl("APP_Save_Button").click()
		self.closeUI()
		
	def setFieldStatus(self, info):
	
		btn = info.find("APP_Save_Button")
		if btn.enabled:
			self.fldFindItem.enable()
		else:
			self.fldFindItem.disable()
		self.fldFindItem.setValue(self.dsD.get("ITEM"))
			
	def fldFindItem_onChange(self, oldvalue, newvalue):
		self.fldFindItem.setValue(newvalue)
		self.goSearchFirst()
		
	def fldFindItem_onClick(self, btn):
	
		if btn == 0:
			self.goSearchFirst()
		elif btn == 1:
			self.goSearchPrev()
		elif btn == 2:
			self.goSearchNext()
		elif btn == 3:
			self.goSearchLast()
		elif btn == 4:
			self.goSearchFinder()
		
	def goSearchFirst(self):

		sFilter = "ITEM = \"{}\"".format(self.fldFindItem.value.rstrip())
		self.dsD.recordClear()
		self.dsD.browse(sFilter, 1)
		if self.dsD.fetch() == 0:
			linenum = self.dsD.get("LINENUM")
		else:
			message("Item [{}] not found in sales order details.".format(self.fldFindItem.value.rstrip()))

	def goSearchLast(self):
		maxline = 9999
		self.dsD.put("LINENUM", maxline, 0)
	
		sFilter = "ITEM = \"{}\"".format(self.fldFindItem.value.rstrip())
		self.dsD.browse(sFilter, 0)
		if self.dsD.fetch() == 0:
			linenum = self.dsD.get("LINENUM")
		else:
			message("Item [{}] not found in sales order details.".format(self.fldFindItem.value.rstrip()))
		
	def goSearchNext(self):
		linenum = self.dsD.get("LINENUM")

		sFilter = "ITEM = \"{}\"".format(self.fldFindItem.value.rstrip())
		self.dsD.browse(sFilter, 1)
		if self.dsD.fetch() == 0:
			if linenum == self.dsD.get("LINENUM"):
				self.dsD.fetch()
		else:
			message("Item [{}] not found in sales order details.".format(self.fldFindItem.value.rstrip()))
		
	def goSearchPrev(self):
		linenum = self.dsD.get("LINENUM")
		
		sFilter = "ITEM = \"{}\"".format(self.fldFindItem.value.rstrip())
		self.dsD.browse(sFilter, 0)
		if self.dsD.fetch() == 0:
			if linenum == self.dsD.get("LINENUM"):
				self.dsD.fetch()
		else:
			message("Item [{}] not found in sales order details.".format(self.fldFindItem.value.rstrip()))
		
	def goSearchFinder(self):
		finder = Finder()
		finder.viewID = "IC0310"
		finder.displayFields = "7,3,1"
		finder.returnFields = "7"	# Return the value field content.
		finder.onOK = self.onSearchFinder_ClickOK
		finder.onCancel = self.onSearchFinder_ClickCancel
		finder.show(self)

	def onSearchFinder_ClickOK(self, e):
		self.fldFindItem.setValue(e)
		self.goSearchFirst()
	
	def onSearchFinder_ClickCancel(self):
		ui_command("return", "1")

